from mbientlab.metawear.core import * 
from mbientlab.metawear.functions import setup_libmetawear
from ctypes import *
import copy
import os
import unittest

class TestMetaWearBase(unittest.TestCase):
    METAWEAR_R_BOARD= 0
    METAWEAR_RG_BOARD= 1
    METAWEAR_RPRO_BOARD= 2
    METAWEAR_CPRO_BOARD= 3
    METAWEAR_ENV_BOARD= 4
    METAWEAR_DETECT_BOARD= 5
    METAWEAR_MOTION_R_BOARD= 6

    @classmethod
    def setUpClass(cls):
        cls.libmetawear= CDLL(os.environ["METAWEAR_LIB_SO_NAME"])
        setup_libmetawear(cls.libmetawear)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.initialized_fn= Fn_VoidPtr_Int(self.initialized)
        self.sensor_data_handler= Fn_DataPtr(self.sensorDataHandler)
        self.timer_signal_ready= Fn_VoidPtr(self.timerSignalReady)
        self.processor_created = Fn_VoidPtr(lambda pointer: self.processors.append(pointer))
        self.logger_created = Fn_VoidPtr(lambda pointer: self.loggers.append(pointer))
        self.commands_recorded_fn= Fn_VoidPtr_Int(self.commandsRecorded)

        self.send_command_fn= Fn_VoidPtr_GattCharPtr_ByteArray(self.commandLogger)
        self.read_gatt_char_fn= Fn_VoidPtr_GattCharPtr(self.read_gatt_char)
        self.btle_connection= BtleConnection(write_gatt_char = self.send_command_fn, read_gatt_char = self.read_gatt_char_fn)

        self.metawear_rg_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2D\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80', 2),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80', 2),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4)
        }

        self.metawear_r_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x00\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x01', 6),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1C', 5),
            0x0a: create_string_buffer(b'\x0A\x80\x00\x00\x1C', 5),
            0x0b: create_string_buffer(b'\x0B\x80\x00\x02\x08\x80\x31\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0C\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0D\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0F\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80', 2),
            0x13: create_string_buffer(b'\x13\x80', 2),
            0x14: create_string_buffer(b'\x14\x80', 2),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xFE\x80\x00\x00', 4)
        }

        self.metawear_rpro_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1C', 5),
            0x0a: create_string_buffer(b'\x0A\x80\x00\x00\x1C', 5),
            0x0b: create_string_buffer(b'\x0B\x80\x00\x02\x08\x80\x2D\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0C\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0D\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0F\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80\x00\x00', 4),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xFE\x80\x00\x00', 4)
        }

        self.metawear_cpro_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1C', 5),
            0x0a: create_string_buffer(b'\x0A\x80\x00\x00\x1C', 5),
            0x0b: create_string_buffer(b'\x0B\x80\x00\x02\x08\x80\x2B\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0C\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0D\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0F\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80\x00\x00', 4),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80\x00\x00', 4),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xFE\x80\x00\x00', 4)
        }

        self.metawear_detector_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x03\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x01\x03\x03\x03\x03\x01', 9),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2b\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x01\x08', 5),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x03', 4),
            0x12: create_string_buffer(b'\x12\x80', 2),
            0x13: create_string_buffer(b'\x13\x80', 2),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80\x00\x00', 4),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4),
        }

        self.metawear_environment_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x03\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x01\x03\x03\x03\x03\x01', 9),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2b\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x01\x08', 5),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x03', 4),
            0x12: create_string_buffer(b'\x12\x80\x01\x00', 4),
            0x13: create_string_buffer(b'\x13\x80', 2),
            0x14: create_string_buffer(b'\x14\x80', 2),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80\x00\x00', 4),
            0x17: create_string_buffer(b'\x17\x80\x00\x00', 4),
            0x18: create_string_buffer(b'\x18\x80', 4),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4),
        }
        self.metawear_motion_r_services= {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x01\x03\x03\x03\x03\x01', 9),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2b\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x01', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x01\x08', 5),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x03', 4),
            0x12: create_string_buffer(b'\x12\x80\x00\x00', 4),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80\x00\x01', 4),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80\x00\x00\x03\x00\x06\x00\x02\x00\x01\x00', 12),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4),
        }

        self.firmware_revision= create_string_buffer(b'1.1.3', 5)

        self.command_history= []
        self.full_history= []

        self.eventId= 0
        self.timerId= 0
        self.dataprocId= 0
        self.loggerId= 0
        self.timerSignals= []
        self.processors= []
        self.loggers= []
        self.boardType= TestMetaWearBase.METAWEAR_R_BOARD
        
    def setUp(self):
        self.board= self.libmetawear.mbl_mw_metawearboard_create(byref(self.btle_connection))
        self.libmetawear.mbl_mw_metawearboard_initialize(self.board, self.initialized_fn)

    def tearDown(self):
        self.libmetawear.mbl_mw_metawearboard_free(self.board)

    def commandsRecorded(self, event, status):
        self.recorded= True

    def timerSignalReady(self, timer_signal):
        self.timerSignals.append(timer_signal)

    def initialized(self, board, status):
        self.init_status= status;

    def read_gatt_char(self, board, characteristic):
        if (characteristic.contents.uuid_high == 0x00002a2400001000 and characteristic.contents.uuid_low == 0x800000805f9b34fb):
            if (self.boardType == TestMetaWearBase.METAWEAR_RG_BOARD):
                model_number= create_string_buffer(b'1', 1)
            elif (self.boardType == TestMetaWearBase.METAWEAR_R_BOARD):
                model_number= create_string_buffer(b'0', 1)
            elif (self.boardType == TestMetaWearBase.METAWEAR_RPRO_BOARD):
                model_number= create_string_buffer(b'1', 1)
            elif (self.boardType == TestMetaWearBase.METAWEAR_CPRO_BOARD or self.boardType == TestMetaWearBase.METAWEAR_DETECT_BOARD or self.boardType == TestMetaWearBase.METAWEAR_ENV_BOARD):
                model_number= create_string_buffer(b'2', 1)
            elif (self.boardType == TestMetaWearBase.METAWEAR_MOTION_R_BOARD):
                model_number= create_string_buffer(b'5', 1)

            self.libmetawear.mbl_mw_connection_char_read(self.board, characteristic, model_number.raw, len(model_number.raw))
        elif (characteristic.contents.uuid_high == 0x00002a2600001000 and characteristic.contents.uuid_low == 0x800000805f9b34fb):
            self.libmetawear.mbl_mw_connection_char_read(self.board, characteristic, self.firmware_revision.raw, len(self.firmware_revision.raw))

    def commandLogger(self, board, characteristic, command, length):
        self.command= []
        for i in range(0, length):
            self.command.append(command[i])

        self.full_history.append(self.command)
        if (command[1] == 0x80):
            if (self.boardType == TestMetaWearBase.METAWEAR_RG_BOARD and command[0] in self.metawear_rg_services):
                service_response= self.metawear_rg_services[command[0]]
            elif (self.boardType == TestMetaWearBase.METAWEAR_R_BOARD and command[0] in self.metawear_r_services):
                service_response= self.metawear_r_services[command[0]]
            elif (self.boardType == TestMetaWearBase.METAWEAR_RPRO_BOARD and command[0] in self.metawear_rpro_services):
                service_response= self.metawear_rpro_services[command[0]]
            elif (self.boardType == TestMetaWearBase.METAWEAR_CPRO_BOARD and command[0] in self.metawear_cpro_services):
                service_response= self.metawear_cpro_services[command[0]]
            elif (self.boardType == TestMetaWearBase.METAWEAR_DETECT_BOARD and command[0] in self.metawear_detector_services):
                service_response= self.metawear_detector_services[command[0]]
            elif (self.boardType == TestMetaWearBase.METAWEAR_ENV_BOARD and command[0] in self.metawear_environment_services):
                service_response= self.metawear_environment_services[command[0]]
            elif (self.boardType == TestMetaWearBase.METAWEAR_MOTION_R_BOARD and command[0] in self.metawear_motion_r_services):
                service_response= self.metawear_motion_r_services[command[0]]

            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, service_response.raw, len(service_response))
        elif (command[0] == 0xb and command[1] == 0x84):
            reference_tick= create_string_buffer(b'\x0b\x84\x15\x04\x00\x00\x05', 7)
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, reference_tick.raw, len(reference_tick))
        else:
            # ignore module discovey commands
            self.command_history.append(self.command)
            if (command[0] == 0xc and command[1] == 0x2):
                response= create_string_buffer(b'\x0c\x02', 3)
                response[2]= self.timerId
                self.timerId+= 1
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
            elif (command[0] == 0xa and command[1] == 0x3):
                response= create_string_buffer(b'\x0a\x02', 3)
                response[2]= self.eventId
                self.eventId+= 1
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
            elif (command[0] == 0x9 and command[1] == 0x2):
                response= create_string_buffer(b'\x09\x02', 3)
                response[2]= self.dataprocId
                self.dataprocId+= 1
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
            elif (command[0] == 0xb and command[1] == 0x2):
                response= create_string_buffer(b'\x0b\x02', 3)
                response[2]= self.loggerId
                self.loggerId+= 1
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
            elif (command[0] == 0xb and command[1] == 0x85):
                response= create_string_buffer(b'\x0b\x85\x9e\x01\x00\x00', 6)
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))

    def sensorDataHandler(self, data):
        if (data.contents.type_id == DataTypeId.UINT32):
            data_ptr= cast(data.contents.value, POINTER(c_uint))
            self.data_uint32= c_uint()
            self.data_uint32.value= data_ptr.contents.value
            self.data = self.data_uint32
        elif (data.contents.type_id == DataTypeId.FLOAT):
            data_ptr= cast(data.contents.value, POINTER(c_float))
            self.data_float= c_float()
            self.data_float.value= data_ptr.contents.value
            self.data = self.data_float
        elif (data.contents.type_id == DataTypeId.CARTESIAN_FLOAT):
            data_ptr= cast(data.contents.value, POINTER(CartesianFloat))
            self.data_cartesian_float= copy.deepcopy(data_ptr.contents)
            self.data = self.data_cartesian_float
        elif (data.contents.type_id == DataTypeId.BATTERY_STATE):
            data_ptr= cast(data.contents.value, POINTER(BatteryState))
            self.data_battery_state= copy.deepcopy(data_ptr.contents)
            self.data = self.data_battery_state
        elif (data.contents.type_id == DataTypeId.BYTE_ARRAY):
            data_ptr= cast(data.contents.value, POINTER(c_ubyte * data.contents.length))

            self.data_byte_array= []
            for i in range(0, data.contents.length):
                self.data_byte_array.append(data_ptr.contents[i])

            self.data = self.data_byte_array
        elif (data.contents.type_id == DataTypeId.TCS34725_ADC):
            data_ptr= cast(data.contents.value, POINTER(Tcs34725ColorAdc))
            self.data_tcs34725_adc= copy.deepcopy(data_ptr.contents)
            self.data = self.data_tcs34725_adc
        elif (data.contents.type_id == DataTypeId.EULER_ANGLES):
            data_ptr= cast(data.contents.value, POINTER(EulerAngle))
            self.data= copy.deepcopy(data_ptr.contents)
        elif (data.contents.type_id == DataTypeId.QUATERNION):
            data_ptr= cast(data.contents.value, POINTER(Quaternion))
            self.data= copy.deepcopy(data_ptr.contents)
        elif (data.contents.type_id == DataTypeId.CORRECTED_CARTESIAN_FLOAT):
            data_ptr= cast(data.contents.value, POINTER(CorrectedCartesianFloat))
            self.data= copy.deepcopy(data_ptr.contents)
        else:
            raise RuntimeError('Unrecognized data type id: ' + str(data.contents.type_id))

    def to_string_buffer(self, bytes):
        buffer= create_string_buffer(len(bytes))
        i= 0
        for b in bytes:
            buffer[i]= b
            i= i + 1

        return buffer



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.sensor import *

class TestDataLength(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD
        self.metawear_environment_services[0x11]= create_string_buffer(b'\x11\x80\x00\x03', 4)
        super().setUp()

    def sensorDataHandler(self, data):
        self.data_length= data.contents.length

    def test_uint32(self):
        expected= 4
        response= create_string_buffer(b'\x05\x87\x01\x72\x03', 5)
    
        pin_monitor_signal= self.libmetawear.mbl_mw_gpio_get_analog_input_data_signal(self.board, 1, Gpio.ANALOG_READ_MODE_ADC);
        self.libmetawear.mbl_mw_datasignal_subscribe(pin_monitor_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, response.raw, len(response))

        self.assertEqual(self.data_length, expected)

    def test_float(self):
        expected= 4
        response= create_string_buffer(b'\x04\x81\x02\xac\xff', 5)

        temp_signal= self.libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(self.board, 
                MultiChannelTemperature.METAWEAR_RPRO_CHANNEL_EXT_THERMISTOR)
        self.libmetawear.mbl_mw_datasignal_subscribe(temp_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, response.raw, len(response))

        self.assertEqual(self.data_length, expected)

    def test_cartesian_float(self):
        expected= 12
        response= create_string_buffer(b'\x03\x04\xe1\xb3\xa1\x24\xb1\x2e', 8)

        accel_data_signal= self.libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(accel_data_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, response.raw, len(response))

        self.assertEqual(self.data_length, expected)

    def test_byte_array(self):
        expected= 1
        response= create_string_buffer(b'\x0d\x81\x0a\x2a', 4)

        signal= self.libmetawear.mbl_mw_i2c_get_data_signal(self.board, 1, 0xa)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, response.raw, len(response.raw))

        self.assertEqual(self.data_length, expected)

    def test_battery_state(self):
        expected= 4
        response= create_string_buffer(b'\x11\x8c\x63\x34\x10', 5)

        signal= self.libmetawear.mbl_mw_settings_get_battery_state_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, response.raw, len(response.raw))

        self.assertEqual(self.data_length, expected)


    def test_tcs34725(self):
        expected= 8
        response= create_string_buffer(b'\x17\x81\xa2\x01\x7b\x00\x9a\x00\x7c\x00', 10)

        signal= self.libmetawear.mbl_mw_cd_tcs34725_get_adc_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_metawearboard_notify_char_changed(self.board, response.raw, len(response.raw))

        self.assertEqual(self.data_length, expected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`led`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from ctypes import POINTER, c_uint, cast
import time

from pymetawear import libmetawear
from pymetawear.client import MetaWearClient
from pymetawear.discover import select_device
from pymetawear.mbientlab.metawear.core import DataTypeId, Fn_DataPtr

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))


def switch_callback(data):
    if (data.contents.type_id == DataTypeId.UINT32):
        data_ptr = cast(data.contents.value, POINTER(c_uint))
        if data_ptr.contents.value == 1:
            print("Switch pressed!")
        elif data_ptr.contents.value == 0:
            print("Switch released!")
        else:
            raise ValueError("Incorrect data returned.")
    else:
        raise RuntimeError('Incorrect data type id: ' + str(data.contents.type_id))


# Subscribe
data_signal = libmetawear.mbl_mw_switch_get_state_data_signal(c.board)
fcn_dptr = Fn_DataPtr(switch_callback)
libmetawear.mbl_mw_datasignal_subscribe(data_signal, fcn_dptr)

time.sleep(10.0)

# Unsubscribe:
libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)

time.sleep(1.0)

c.disconnect()




# -*- coding: utf-8 -*-
"""Release data for the PyMetaWear project."""

# -----------------------------------------------------------------------------
#  Copyright (c) 2016, Nedomkull Mathematical Modeling AB.
# -----------------------------------------------------------------------------

import os
import sys
import platform
import glob
from ctypes import cdll

from pymetawear.version import __version__, version  # flake8: noqa
from pymetawear.mbientlab.metawear.functions import setup_libmetawear

# Logging solution inspired by Hitchhiker's Guide to Python and Requests
# Set default logging handler to avoid "No handler found" warnings.
import logging
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())


# Find and import the built MetaWear-CPP shared library.
if os.environ.get('METAWEAR_LIB_SO_NAME') is not None:
    METAWEAR_LIB_SO_NAME = os.environ["METAWEAR_LIB_SO_NAME"]
else:
    if platform.uname()[0] == 'Windows':
        dll_files = list(glob.glob(os.path.join(os.path.abspath(
            os.path.dirname(__file__)), 'MetaWear.*.dll')))
        METAWEAR_LIB_SO_NAME = dll_files[0]
    else:
        METAWEAR_LIB_SO_NAME = 'libmetawear.so'
    METAWEAR_LIB_SO_NAME = os.path.join(os.path.abspath(
        os.path.dirname(__file__)), METAWEAR_LIB_SO_NAME)
libmetawear = cdll.LoadLibrary(METAWEAR_LIB_SO_NAME)
setup_libmetawear(libmetawear)


def add_stream_logger(stream=sys.stdout, level=logging.DEBUG):
    """
    Helper for quickly adding a StreamHandler to the logger. Useful for
    debugging.
    Returns the handler after adding it.
    """
    logger = logging.getLogger(__name__)
    has_stream_handler = any([isinstance(hndl, logging.StreamHandler) for hndl in logger.handlers])
    if has_stream_handler:
        return logger.handlers[-1]
    handler = logging.StreamHandler(stream=stream)
    handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s %(message)s'))
    logger.addHandler(handler)
    logger.setLevel(level)
    logger.debug('Added a {0} logging handler to logger: {1}'.format(stream.name, __name__))
    return handler



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging
import warnings
from functools import wraps
from ctypes import cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import DataTypeId, BatteryState
from pymetawear.modules.base import PyMetaWearModule

log = logging.getLogger(__name__)


class BatteryModule(PyMetaWearModule):
    """MetaWear Settings/Battery module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, debug=False):
        super(BatteryModule, self).__init__(board, debug)

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}: {1}".format(self.module_name, self.sensor_name)

    def __repr__(self):
        return super(BatteryModule, self).__repr__()

    @property
    def module_name(self):
        return 'Settings'

    @property
    def sensor_name(self):
        return 'Battery'

    @property
    def data_signal(self):
        return libmetawear.mbl_mw_settings_get_battery_state_data_signal(self.board)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to battery notifications.

        Convenience method for handling battery notifications.

        The data to the callback method comes as a tuple of two integer
        values, the first one representing the voltage and the second one
        is an integer in [0, 100] representing battery percentage.

        Example:

        .. code-block:: python

            def battery_callback(data):
                epoch = data[0]
                battery = data[1]
                print("[{0}] Voltage: {1}, Charge: {2}".format(
                    epoch, battery[0], battery[1]))

            mwclient.battery.notifications(battery_callback)
            mwclient.battery.read_battery_state()

        :param callable callback: Battery data notification callback
            function. If `None`, unsubscription to battery notifications
            is registered.

        """
        super(BatteryModule, self).notifications(
            battery_data(callback) if callback is not None else None)

    def read_battery_state(self):
        """Triggers a battery state notification.

        N.B. that a :meth:`~notifications` call that registers a
        callback for battery state should have been done prior to calling
        this method.

        """
        if self.callback is None:
            warnings.warn("No battery callback is registered!", RuntimeWarning)
        libmetawear.mbl_mw_datasignal_read(self.data_signal)


def battery_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.BATTERY_STATE:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(BatteryState))
            func((epoch, (int(data_ptr.contents.voltage),
                          int(data_ptr.contents.charge))))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))
    return wrapper



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.core import CartesianFloat
from mbientlab.metawear.sensor import MagnetometerBmm150

class TestMagnetometerBmm150(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_CPRO_BOARD

        super().setUp()

    def test_power_preset(self):
        tests= [
            {
                'expected': [[0x15, 0x04, 0x01, 0x02], [0x15, 0x03, 0x0]] ,
                'preset': MagnetometerBmm150.PRESET_LOW_POWER,
                'preset_name': 'low power',
                'offset': 0
            },
            {
                'expected': [[0x15, 0x04, 0x04, 0x0e], [0x15, 0x03, 0x00]], 
                'preset': MagnetometerBmm150.PRESET_REGULAR,
                'preset_name': 'regular',
                'offset': 2
            },
            {
                'expected': [[0x15, 0x04, 0x07, 0x1a], [0x15, 0x03, 0x00]], 
                'preset': MagnetometerBmm150.PRESET_ENHANCED_REGULAR,
                'preset_name': 'enhanced regular',
                'offset': 4
            },
            {
                'expected': [[0x15, 0x04, 0x17, 0x52], [0x15, 0x03, 0x05]], 
                'preset': MagnetometerBmm150.PRESET_HIGH_ACCURACY,
                'preset_name': 'high accuracy',
                'offset': 6
            },
        ]

        for test in tests:
            with self.subTest(preset= test['preset_name']):
                offset= test['offset']
                self.libmetawear.mbl_mw_mag_bmm150_set_preset(self.board, test['preset'])
                self.assertEqual(self.command_history[offset:offset + 2], test['expected'])

    def test_enable_b_field_sampling(self):
        expected= [0x15, 0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_mag_bmm150_enable_b_field_sampling(self.board)
        self.assertEqual(self.command, expected)

    def test_disable_b_field_sampling(self):
        expected= [0x15, 0x02, 0x00, 0x01]

        self.libmetawear.mbl_mw_mag_bmm150_disable_b_field_sampling(self.board)
        self.assertEqual(self.command, expected)

    def test_start(self):
        expected= [0x15, 0x01, 0x01]

        self.libmetawear.mbl_mw_mag_bmm150_start(self.board)
        self.assertEqual(self.command, expected)

    def test_stop(self):
        expected= [0x15, 0x01, 0x00]

        self.libmetawear.mbl_mw_mag_bmm150_stop(self.board)
        self.assertEqual(self.command, expected)

class TestMagnetometerBmm150Data(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_CPRO_BOARD

        super().setUp()

    def test_b_field_subscribe(self):
        expected= [0x15, 0x05, 0x01]

        signal= self.libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)

    def test_b_field_component_subscribe(self):
        # expect only 1 subscribe & unsubscribe commands to be issued
        expected_cmds= [
            [0x15, 0x05, 0x01],
            [0x15, 0x05, 0x00]
        ]

        signal= self.libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)

        indices= [MagnetometerBmm150.BFIELD_X_AXIS_INDEX, MagnetometerBmm150.BFIELD_Y_AXIS_INDEX, MagnetometerBmm150.BFIELD_Z_AXIS_INDEX]
        for i in indices:
            component= self.libmetawear.mbl_mw_datasignal_get_component(signal, i)
            self.libmetawear.mbl_mw_datasignal_subscribe(component, self.sensor_data_handler)

        for i in indices:
            component= self.libmetawear.mbl_mw_datasignal_get_component(signal, i)
            self.libmetawear.mbl_mw_datasignal_unsubscribe(component)

        self.libmetawear.mbl_mw_datasignal_unsubscribe(signal)

        self.assertEqual(self.command_history, expected_cmds)

    def test_b_field_unsubscribe(self):
        expected= [0x15, 0x05, 0x00]

        signal= self.libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(signal)

    def test_b_field_data(self):
        expected= CartesianFloat(x = -251.1250, y = 165.1875, z = 71.3125)
        response= create_string_buffer(b'\x15\x05\x4e\xf0\x53\x0a\x75\x04', 8)

        signal= self.libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))

        self.assertEqual(self.data_cartesian_float, expected)

    def test_b_field_component_data(self):
        response= create_string_buffer(b'\x15\x05\x4e\xf0\x53\x0a\x75\x04', 8)
        tests= [
            {
                'expected': -251.1250,
                'index': MagnetometerBmm150.BFIELD_X_AXIS_INDEX,
                'name': 'x-axis'
            },
            {
                'expected': 165.1875,
                'index': MagnetometerBmm150.BFIELD_Y_AXIS_INDEX,
                'name': 'y-axis'
            },
            {
                'expected': 71.3125,
                'index': MagnetometerBmm150.BFIELD_Z_AXIS_INDEX,
                'name': 'z-axis'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                signal= self.libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)
                signal_component = self.libmetawear.mbl_mw_datasignal_get_component(signal, test['index'])
                self.libmetawear.mbl_mw_datasignal_subscribe(signal_component, self.sensor_data_handler)
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))

                self.assertAlmostEqual(self.data_float.value, test['expected'], delta = 0.001)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@swedwise.com>

Created on 2016-04-08, 14:01

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import


try:
    # Python 2
    range_ = xrange
    string_types = (basestring, )

    def bytearray_to_str(ba):
        return str(ba)

except NameError:
    # Python 3
    range_ = range
    string_types = (str, )

    def bytearray_to_str(ba):
        if isinstance(ba, string_types):
            # PyBluez
            return ba.encode()
        else:
            # PyGatt
            return bytes([x for x in ba])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-28

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import copy
import logging
from functools import wraps
from ctypes import c_uint, cast, POINTER, c_float, c_ubyte

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import DataTypeId, CartesianFloat, BatteryState, Tcs34725ColorAdc
from pymetawear.modules.base import PyMetaWearModule, Modules
from pymetawear.mbientlab.metawear.sensor import AmbientLightLtr329

log = logging.getLogger(__name__)


def require_ltr329(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'ambient_light_class', None) is None:
            raise PyMetaWearException("There is not Ambient Light "
                                      "module on your MetaWear board!")
        return f(*args, **kwargs)
    return wrapper


class AmbientLightModule(PyMetaWearModule):
    """MetaWear Ambient Light module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id, debug=False):
        super(AmbientLightModule, self).__init__(board, debug)
        self.module_id = module_id

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No ambient light sensor present!
            self.ambient_light_class = None
            self.is_present = False
        else:
            self.ambient_light_class = AmbientLightLtr329

        if self.ambient_light_class is not None:
            # Parse possible output data rates for this accelerometer.
            self._gain = {
                int(re.search('^GAIN_([0-9]+)X', k).groups()[0]):
                    getattr(self.ambient_light_class, k, None) for k in
                filter(lambda x: x.startswith('GAIN'),
                       vars(self.ambient_light_class))}
            self._integration_time = {
                int(re.search('^INTEGRATION_TIME_([0-9]+)MS', k).groups()[0]):
                    getattr(self.ambient_light_class, k, None) for k in
                filter(lambda x: x.startswith('INTEGRATION_TIME'),
                       vars(self.ambient_light_class))}
            self._measurement_rate = {
                int(re.search('^MEASUREMENT_RATE_([0-9]+)MS', k).groups()[0]):
                    getattr(self.ambient_light_class, k, None) for k in
                filter(lambda x: x.startswith('MEASUREMENT_RATE'),
                       vars(self.ambient_light_class))}

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}".format(
            self.module_name)

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Ambient Light"

    @property
    @require_ltr329
    def data_signal(self):
        return libmetawear.mbl_mw_als_ltr329_get_illuminance_data_signal(self.board)

    @require_ltr329
    def get_current_settings(self):
        raise NotImplementedError()

    @require_ltr329
    def get_possible_settings(self):
        return {
            'gain': [x for x in sorted(self._gain.keys())],
            'integration_time': [x for x in sorted(
                self._integration_time.keys())],
            'measurement_rate': [x for x in sorted(
                self._measurement_rate.keys())]
        }

    def _get_gain(self, value):
        sorted_ord_keys = sorted(self._gain.keys(),
                                 key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested gain ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        k = int(sorted_ord_keys[diffs.index(min_diffs)])
        return self._gain.get(k)

    def _get_integration_time(self, value):
        sorted_ord_keys = sorted(self._integration_time.keys(),
                                 key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested integration time ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        k = int(sorted_ord_keys[diffs.index(min_diffs)])
        return self._integration_time.get(k)

    def _get_measurement_rate(self, value):
        sorted_ord_keys = sorted(self._measurement_rate.keys(),
                                 key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested measurement rate ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        k = int(sorted_ord_keys[diffs.index(min_diffs)])
        return self._measurement_rate.get(k)

    @require_ltr329
    def set_settings(self, gain=None, integration_time=None,
                     measurement_rate=None):
        """Set ambient light sensor settings.

         Can be called with two or only one setting:

         .. code-block:: python

            mwclient.ambient_light.set_settings(
                gain=4,
                integration_time=200,
                measurement_rate=200)

        will give the same result as

        .. code-block:: python

            mwclient.ambient_light.set_settings(gain=4)
            mwclient.ambient_light.set_settings(integration_time=200)
            mwclient.ambient_light.set_settings(measurement_rate=200)

        albeit that the latter example makes three writes to the board.

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param float gain: Sensor gain
        :param float integration_time: Sensor integration time
        :param float measurement_rate: Sensor measurement rate

        """
        if gain is not None:
            g = self._get_gain(gain)
            if self._debug:
                log.debug("Setting Ambient Light gain to {0}".format(g))
            libmetawear.mbl_mw_als_ltr329_set_gain(self.board, g)
        if integration_time is not None:
            itime = self._get_integration_time(integration_time)
            if self._debug:
                log.debug("Setting Ambient Light integration time to {0}".format(itime))
            libmetawear.mbl_mw_als_ltr329_set_integration_time(self.board, itime)
        if measurement_rate is not None:
            mr = self._get_measurement_rate(measurement_rate)
            if self._debug:
                log.debug("Setting Ambient Light measurement rate to {0}".format(mr))
            libmetawear.mbl_mw_als_ltr329_set_measurement_rate(self.board, mr)

        if (gain is not None) or (integration_time is not None) or \
                (measurement_rate is not None):
            libmetawear.mbl_mw_als_ltr329_write_config(self.board)

    @require_ltr329
    def notifications(self, callback=None):
        """Subscribe or unsubscribe to notifications.

        Convenience method for handling ambient light sensor usage.

        Example:

        .. code-block:: python

            def al_callback(data):
                print(data)

            mwclient.ambient_light.notifications(al_callback)

        :param callable callback: Ambient Light notification callback function.
            If `None`, unsubscription to ambient light notifications
            is registered.

        """
        if callback is None:
            super(AmbientLightModule, self).notifications(None)
            self.stop()
        else:
            super(AmbientLightModule, self).notifications(
                ambient_light_data(callback))
            self.start()

    @require_ltr329
    def start(self):
        """Starts luminance sampling"""
        libmetawear.mbl_mw_als_ltr329_start(self.board)

    @require_ltr329
    def stop(self):
        """Stops luminance sampling"""
        libmetawear.mbl_mw_als_ltr329_stop(self.board)


def ambient_light_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.UINT32:
            data_ptr = cast(data.contents.value, POINTER(c_uint))
            func(int(data_ptr.contents.value))
        elif data.contents.type_id == DataTypeId.FLOAT:
            data_ptr = cast(data.contents.value, POINTER(c_float))
            func(float(data_ptr.contents.value))
        elif data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
            data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
            func((data_ptr.contents.x,
                  data_ptr.contents.y,
                  data_ptr.contents.z))
        elif data.contents.type_id == DataTypeId.BATTERY_STATE:
            data_ptr = cast(data.contents.value, POINTER(BatteryState))
            func((int(data_ptr.contents.voltage),
                  int(data_ptr.contents.charge)))
        elif data.contents.type_id == DataTypeId.BYTE_ARRAY:
            data_ptr = cast(data.contents.value,
                            POINTER(c_ubyte * data.contents.length))
            data_byte_array = []
            for i in range(0, data.contents.length):
                data_byte_array.append(int(data_ptr.contents[i]))
            func(data_byte_array)
        elif data.contents.type_id == DataTypeId.TCS34725_ADC:
            data_ptr = cast(data.contents.value, POINTER(Tcs34725ColorAdc))
            data_tcs34725_adc = copy.deepcopy(data_ptr.contents)
            func(data_tcs34725_adc)
        else:
            raise RuntimeError(
                'Unrecognized data type id: ' + str(data.contents.type_id))

    return wrapper



from common import TestMetaWearBase
from mbientlab.metawear.core import CartesianFloat
from mbientlab.metawear.sensor import Accelerometer, AccelerometerMma8452q
from ctypes import create_string_buffer

class TestMma8452qConfiguration(TestMetaWearBase):
    def test_set_odr(self):
        expected= [0x03, 0x03, 0x00, 0x00, 0x20, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_set_odr(self.board, AccelerometerMma8452q.ODR_50HZ)
        self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_range(self):
        expected= [0x03, 0x03, 0x02, 0x00, 0x18, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_set_range(self.board, AccelerometerMma8452q.FSR_8G)
        self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_set_odr_and_range(self):
        expected= [0x03, 0x03, 0x01, 0x00, 0x10, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_set_odr(self.board, AccelerometerMma8452q.ODR_200HZ)
        self.libmetawear.mbl_mw_acc_mma8452q_set_range(self.board, AccelerometerMma8452q.FSR_4G)
        self.libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01]

        self.libmetawear.mbl_mw_acc_mma8452q_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00]

        self.libmetawear.mbl_mw_acc_mma8452q_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

class TestAccMma8452qAccelerationData(TestMetaWearBase):
    def setUp(self):
        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal(self.board)

    def test_get_acceleration_data_g(self):
        response= create_string_buffer(b'\x03\x04\x56\xfa\x05\xf6\x18\x03', 8)
        expected= CartesianFloat(x= -1.450, y= -2.555, z= 0.792)

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))

        self.assertEqual(self.data_cartesian_float, expected)

    def test_handle_data_component(self):
        response= create_string_buffer(b'\x03\x04\x56\xfa\x05\xf6\x18\x03', 8)
        tests= [
            {
                'expected': -1.450,
                'index': Accelerometer.ACCEL_X_AXIS_INDEX,
                'name': 'x-axis'
            },
            {
                'expected': -2.555,
                'index': Accelerometer.ACCEL_Y_AXIS_INDEX,
                'name': 'y-axis'
            },
            {
                'expected': 0.792,
                'index': Accelerometer.ACCEL_Z_AXIS_INDEX,
                'name': 'z-axis'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                acc_component = self.libmetawear.mbl_mw_datasignal_get_component(self.accel_data_signal, test['index'])
                self.libmetawear.mbl_mw_datasignal_subscribe(acc_component, self.sensor_data_handler)
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))
                
                self.assertAlmostEqual(self.data_float.value, test['expected'], delta = 0.001)

    def test_stream_acceleration_data(self):
        expected= [0x03, 0x04, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_end_stream_acceleration_data(self):
        expected= [0x03, 0x04, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)

class TestAccMma8452qHighFreqAccData(TestMetaWearBase):
    def setUp(self):
        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_mma8452q_get_high_freq_acceleration_data_signal(self.board)

    def sensorDataHandler(self, data):
        super().sensorDataHandler(data)

        self.cartesian_float_values.append(self.data_cartesian_float)

    def test_get_acceleration_data_g(self):
        response= create_string_buffer(b'\x03\x12\xdf\xed\x2b\x16\x15\xff\x7d\xfa\xed\x04\xc5\x0a\x31\xfb\xdb\xf8\x7b\xf2', 20)
        expected_values= [CartesianFloat(x= -4.641, y= 5.675, z= -0.235), CartesianFloat(x= -1.411, y= 1.261, z= 2.757), CartesianFloat(x= -1.231, y= -1.829, z= -3.461)]

        self.cartesian_float_values= []
        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))

        self.assertEqual(self.cartesian_float_values, expected_values)

    def test_stream_acceleration_data(self):
        expected= [0x03, 0x12, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_end_stream_acceleration_data(self):
        expected= [0x03, 0x12, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PyBlueZ backend
---------------

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
import uuid
from ctypes import create_string_buffer
import logging

try:
    from bluetooth.ble import GATTRequester, GATTResponse
    _import_failure = None
except ImportError as e:
    GATTRequester = object
    _import_failure = e

from pymetawear.exceptions import PyMetaWearException, PyMetaWearConnectionTimeout
from pymetawear.compat import range_, string_types, bytearray_to_str
from pymetawear.backends import BLECommunicationBackend

__all__ = ["PyBluezBackend"]

log = logging.getLogger(__name__)


class Requester(GATTRequester):

    def __init__(self, notify_fcn, *args):
        super(Requester, self).__init__(*args)
        self.notify_fcn = notify_fcn

    def on_notification(self, handle, data):
        return self.notify_fcn(handle, data[3:])


class PyBluezBackend(BLECommunicationBackend):
    """
    Backend using `pybluez <https://github.com/karulis/pybluez>`_ and
    `gattlib <https://bitbucket.org/OscarAcena/pygattlib>`_ for BLE communication.
    """

    def __init__(self, address, interface=None, timeout=None, debug=False):
        if _import_failure is not None:
            raise PyMetaWearException(
                "pybluez[ble] package error: {0}".format(_import_failure))
        self.name = 'pybluez/gattlib'
        self._primary_services = {}
        self._characteristics_cache = {}
        self._response = GATTResponse()
        if debug:
            log.setLevel(logging.DEBUG)

        super(PyBluezBackend, self).__init__(
            address, interface, 10.0 if timeout is None else timeout, debug)

    def _build_handle_dict(self):
        self._primary_services = {uuid.UUID(x.get('uuid')): (x.get('start'), x.get('end'))
                                  for x in self.requester.discover_primary()}
        self._characteristics_cache = {uuid.UUID(x.get('uuid')): (x.get('value_handle'), x.get('value_handle') + 1)
                                       for x in self.requester.discover_characteristics()}

    @property
    def is_connected(self):
        if self._requester is not None:
            return self._requester.is_connected()
        return False

    @property
    def requester(self):
        """Property handling `GattRequester` and its connection.

        :return: The connected GattRequester instance.
        :rtype: :class:`bluetooth.ble.GATTRequester`

        """
        if self._requester is None:
            self.connect()

        if not self.is_connected:
            raise PyMetaWearConnectionTimeout(
                "PyBluezBackend: Connection to {0} lost...".format(self._address))

        return self._requester

    def connect(self, clean_connect=False):
        if self.is_connected:
            return

        if clean_connect or self._requester is None:
            log.info("PyBluezBackend: Creating new GATTRequester...")
            self._requester = Requester(self.handle_notify_char_output,
                                        self._address,
                                        False, self._interface)

        log.info("PyBluezBackend: Connecting GATTRequester...")
        self._requester.connect(wait=False, channel_type='random')
        # Using manual waiting since gattlib's `wait` keyword does not work.
        t = 0.0
        t_step = 0.25
        while not self._requester.is_connected() and t < self._timeout:
            t += t_step
            time.sleep(t_step)

        if not self._requester.is_connected():
            raise PyMetaWearConnectionTimeout(
                "PyBluezBackend: Could not establish a connection to {0}.".format(self._address))

        super(PyBluezBackend, self).connect()

    def disconnect(self):
        """Disconnect."""
        if self._requester is not None and self._requester.is_connected():
            self._requester.disconnect()
            self._requester = None

    def _subscribe(self, characteristic_uuid, callback):
        # Subscribe to Notify Characteristic.
        handle = self.get_handle(characteristic_uuid, notify_handle=True)
        bytes_to_send = bytearray([0x01, 0x00])
        return self.requester.write_by_handle_async(
            handle, bytes(bytes_to_send), self._response)

    # Read and Write methods

    def read_gatt_char_by_uuid(self, characteristic):
        """Read the desired data from the MetaWear board
        using pybluez/gattlib backend.

        :param uuid.UUID characteristic_uuid: Characteristic UUID to read from.
        :return: The read data.
        :rtype: str

        """
        characteristic_uuid = self.get_uuid(characteristic)
        return self.requester.read_by_uuid(str(characteristic_uuid))[0]

    def write_gatt_char_by_uuid(self, characteristic, command, length):
        """Write the desired data to the MetaWear board
        using pybluez/gattlib backend.

        :param uuid.UUID characteristic_uuid: Characteristic UUID to write to.
        :param str data_to_send: Data to send.

        """
        characteristic_uuid = self.get_uuid(characteristic)
        handle = self.get_handle(characteristic_uuid)
        data_to_send = bytes(bytearray([command[i] for i in range_(length)]))
        if not isinstance(data_to_send, string_types):
            data_to_send = data_to_send.decode('latin1')
        self.requester.write_by_handle_async(handle, data_to_send, self._response)

    def get_handle(self, characteristic_uuid, notify_handle=False):
        """Get handle for a characteristic UUID.

        :param uuid.UUID characteristic_uuid: The UUID for the characteristic to look up.
        :param bool notify_handle:
        :return: The handle for this UUID.
        :rtype: int

        """
        if isinstance(characteristic_uuid, string_types):
            characteristic_uuid = characteristic_uuid.UUID(
                characteristic_uuid)
        handle = self._characteristics_cache.get(
            characteristic_uuid, [None, None])[int(notify_handle)]
        if handle is None:
            raise PyMetaWearException("Incorrect characteristic.")
        else:
            return handle

    def _response_2_string_buffer(self, response):
        return create_string_buffer(bytearray_to_str(response), len(response))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`battery`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
from math import atan2, pi

import matplotlib.pyplot as plt

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient


class ComplimentaryFilter(object):
    """A simple, not very efficient Complimentary Filter example.

    A ``tau``-value of 0.075 leads to a pretty good filtering.

    Args:

        tau (float): Response coefficientS. Higher value leads to more
            gyro-driven filtering.
        f (int): Frequency of sensor sampling.

    References:

    http://www.pieter-jan.com/node/11
    http://robottini.altervista.org/tag/complementary-filter

    """

    def __init__(self, tau, f):
        """Constructor"""

        self.frequency = f
        self._dt = 1.0 / self.frequency
        self.k = tau / (self._dt + tau)

        self._acc_data_added = False
        self._gyro_data_added = False

        self.raw_acc_data = []
        self.raw_gyro_data = []
        self.unfiltered_data = []
        self.filtered_data = []
        self.filtered_data_timestamps = []

    def get_pitch_roll_from_accelerometer(self, acc):
        pitch = atan2(acc[1], acc[2]) * 180 / pi
        roll = atan2(acc[0], acc[2]) * 180 / pi
        return pitch, roll

    def update_angle(self, acc, gyro):
        old_pitch, old_roll = self.filtered_data[-1]
        acc_pitch, acc_roll = self.get_pitch_roll_from_accelerometer(acc)
        self.unfiltered_data.append((acc_pitch, acc_roll))
        new_pitch = self.k * (old_pitch + gyro[0] * self._dt) + (1 - self.k) * acc_pitch
        new_roll = self.k * (old_roll + gyro[1] * self._dt) + (1 - self.k) * acc_roll
        return new_pitch, new_roll

    def add_accelerometer_data(self, data):
        if not self.raw_acc_data:
            self.filtered_data.append(self.get_pitch_roll_from_accelerometer(data[1]))
        self.raw_acc_data.append(data)
        self._acc_data_added = True
        if self._acc_data_added and self._gyro_data_added:
            self._acc_data_added = False
            self._gyro_data_added = False
            self.filtered_data.append(self.update_angle(
                self.raw_acc_data[-1][1], self.raw_gyro_data[-1][1]))
            self.filtered_data_timestamps.append(self.raw_acc_data[-1][0])

    def add_gyroscope_data(self, data):
        self.raw_gyro_data.append(data)
        self._gyro_data_added = True
        if self._acc_data_added and self._gyro_data_added:
            self._acc_data_added = False
            self._gyro_data_added = False
            self.filtered_data.append(self.update_angle(
                self.raw_acc_data[-1][1], self.raw_gyro_data[-1][1]))
            self.filtered_data_timestamps.append(self.raw_gyro_data[-1][0])

    def plot(self):
        timestamps = [ts - self.filtered_data_timestamps[0] for ts in self.filtered_data_timestamps]

        ax = plt.subplot(211)
        ax.set_title("Pitch")
        ax.plot(timestamps, [x[0] for x in self.filtered_data[1:]])
        ax.plot(timestamps, [x[0] for x in self.unfiltered_data], 'r--')
        ax.legend(['Filtered pitch', 'Raw pitch'])

        ax = plt.subplot(212)
        ax.set_title("Roll")
        ax.plot(timestamps, [x[1] for x in self.filtered_data[1:]])
        ax.plot(timestamps, [x[1] for x in self.unfiltered_data], 'r--')
        ax.legend(['Filtered roll', 'Raw roll'])

        plt.show()


def run():
    """Example of how to run the Complimentary filter."""
    address = select_device()
    c = MetaWearClient(str(address), 'pygatt', timeout=10, debug=False)
    print("New client created: {0}".format(c))
    f = ComplimentaryFilter(0.075, 50.0)

    print("Write accelerometer settings...")
    c.accelerometer.set_settings(data_rate=50.0, data_range=4.0)
    c.gyroscope.set_settings(data_rate=50.0, data_range=250.0)
    print("Subscribing to accelerometer signal notifications...")
    c.accelerometer.high_frequency_stream = False
    c.accelerometer.notifications(f.add_accelerometer_data)
    c.gyroscope.notifications(f.add_gyroscope_data)
    time.sleep(10.0)

    print("Unsubscribe to notification...")
    c.accelerometer.notifications(None)
    c.gyroscope.notifications(None)

    c.disconnect()

    return f


if __name__ == '__main__':
    f = run()
    f.plot()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`led`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))

print("Blinking 10 times with green LED...")
pattern = c.led.load_preset_pattern('blink', repeat_count=10)
c.led.write_pattern(pattern, 'g')
c.led.play()

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`modules`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from .base import PyMetaWearModule, Modules
from .accelerometer import AccelerometerModule
from .ambientlight import AmbientLightModule
from .barometer import BarometerModule
from .battery import BatteryModule
from .gyroscope import GyroscopeModule
from .haptic import HapticModule
from .led import LEDModule
from .magnetometer import MagnetometerModule
from .switch import SwitchModule
from .temperature import TemperatureModule
from .sensorfusion import SensorFusionModule

__all__ = [
    "PyMetaWearModule", "Modules",
    "AccelerometerModule", "AmbientLightModule",
    "BarometerModule", "BatteryModule",
    "GyroscopeModule", "HapticModule",
    "LEDModule", "MagnetometerModule",
    "SwitchModule", "TemperatureModule",
    "SensorFusionModule"
]



from common import TestMetaWearBase

class TestGenericAccelerometerBmi160(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RG_BOARD

        super().setUp()

    def test_acc_bmi160_set_odr_cmd(self):
        expected= [0x03, 0x03, 0x27, 0x03]

        self.libmetawear.mbl_mw_acc_set_odr(self.board, 55.0)
        self.libmetawear.mbl_mw_acc_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bmi160_set_odr_value(self):
        expected= 50.0

        actual_odr= self.libmetawear.mbl_mw_acc_set_odr(self.board, 55.0)
        self.assertAlmostEqual(actual_odr, expected)

    def test_acc_bmi160_set_range_cmd(self):
        expected= [0x03, 0x03, 0x28, 0x0c]
 
        self.libmetawear.mbl_mw_acc_set_range(self.board, 14.75)
        self.libmetawear.mbl_mw_acc_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bmi160_set_range_value(self):
        expected= 16.0 

        actual_range= self.libmetawear.mbl_mw_acc_set_range(self.board, 14.75)
        self.assertAlmostEqual(actual_range, expected)

    def test_acc_bmi160_subscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x01]

        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_acc_bmi160_unsubscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x00]

        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(signal, self.sensor_data_handler)
        self.assertAlmostEqual(self.command, expected)

    def test_acc_bmi160_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_acc_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bmi160_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00, 0x01]

        self.libmetawear.mbl_mw_acc_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_module_null(self):
        result= self.libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal(self.board)
        self.assertEqual(result, None)

    def test_acc_bosch_module_valid(self):
        result= self.libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal(self.board)
        self.assertNotEqual(result, None)

class TestGenericAccelerometerMma8452q(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_R_BOARD
        super().setUp()

    def test_acc_mma8452q_set_odr_cmd(self):
        expected= [0x03, 0x03, 0x00, 0x00, 0x20, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_set_odr(self.board, 35.25)
        self.libmetawear.mbl_mw_acc_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_set_odr_value(self):
        expected= 50.0 

        actual_odr= self.libmetawear.mbl_mw_acc_set_odr(self.board, 35.25)
        self.assertAlmostEqual(actual_odr, expected)

    def test_acc_mma8452q_set_range_cmd(self):
        expected= [0x03, 0x03, 0x02, 0x00, 0x18, 0x00, 0x00]

        self.libmetawear.mbl_mw_acc_set_range(self.board, 7.3333)
        self.libmetawear.mbl_mw_acc_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_set_range_value(self):
        expected= 8.0

        actual_range= self.libmetawear.mbl_mw_acc_set_range(self.board, 7.3333)
        self.assertAlmostEqual(actual_range, expected)

    def test_acc_mma8452q_subscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x01]

        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_unsubscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x00]

        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01]

        self.libmetawear.mbl_mw_acc_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00]

        self.libmetawear.mbl_mw_acc_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bosch_module_null(self):
        result= self.libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal(self.board)
        self.assertEqual(result, None)

    def test_acc_mma8452q_module_valid(self):
        result= self.libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal(self.board)
        self.assertNotEqual(result, None)

class TestGenericAccelerometerBma255(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

    def test_acc_bma255_set_odr_cmd(self):
        expected= [0x03, 0x03, 0x0a, 0x03]

        self.libmetawear.mbl_mw_acc_set_odr(self.board, 50.0)
        self.libmetawear.mbl_mw_acc_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bma255_set_odr_value(self):
        expected= 62.5

        actual_odr= self.libmetawear.mbl_mw_acc_set_odr(self.board, 50.0)
        self.assertAlmostEqual(actual_odr, expected)

    def test_acc_bma255_set_range_cmd(self):
        expected= [0x03, 0x03, 0xa, 0x0c]
 
        self.libmetawear.mbl_mw_acc_set_range(self.board, 20.0)
        self.libmetawear.mbl_mw_acc_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bma255_set_range_value(self):
        expected= 16.0

        actual_range= self.libmetawear.mbl_mw_acc_set_range(self.board, 20.0)
        self.assertAlmostEqual(actual_range, expected)

    def test_acc_bma255_subscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x01]

        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_acc_bma255_unsubscribe_acceleration_data(self):
        expected= [0x03, 0x04, 0x00]

        signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_unsubscribe(signal, self.sensor_data_handler)
        self.assertAlmostEqual(self.command, expected)

    def test_acc_bma255_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_acc_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_bma255_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00, 0x01]

        self.libmetawear.mbl_mw_acc_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_acc_mma8452q_module_null(self):
        result= self.libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal(self.board)
        self.assertEqual(result, None)


    def test_acc_bosch_module_valid(self):
        result= self.libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal(self.board)
        self.assertNotEqual(result, None)



from ctypes import c_int, c_ushort, c_ubyte, c_void_p, Structure

class Accelerometer:
    ACCEL_X_AXIS_INDEX = 0
    ACCEL_Y_AXIS_INDEX = 1
    ACCEL_Z_AXIS_INDEX = 2

class AccelerometerBosch:
    FSR_2G= 0
    FSR_4G= 1
    FSR_8G= 2
    FSR_16G= 3

# Constants for configuring the BMI160 accelerometer
class AccelerometerBmi160:
    MODULE_TYPE= 1

    ODR_0_78125HZ= 0
    ODR_1_5625HZ= 1
    ODR_3_125HZ= 2
    ODR_6_25HZ= 3
    ODR_12_5HZ= 4
    ODR_25HZ= 5
    ODR_50HZ= 6
    ODR_100HZ= 7
    ODR_200HZ= 8
    ODR_400HZ= 9
    ODR_800HZ= 10
    ODR_1600HZ= 11

    STEP_COUNTER_MODE_NORMAL= 0
    STEP_COUNTER_MODE_SENSITIVE= 1
    STEP_COUNTER_MODE_ROBUST= 2

class AccelerometerBma255:
    MODULE_TYPE= 3

    ODR_15_62HZ= 0
    ODR_31_26HZ= 1
    ODR_62_5HZ= 2
    ODR_125HZ= 3
    ODR_250HZ= 4
    ODR_500HZ= 5
    ODR_1000HZ= 6
    ODR_2000HZ= 7

# Constants for configuring the MMA8452Q accelerometer
class AccelerometerMma8452q:
    MODULE_TYPE= 0

    ODR_800HZ= 0
    ODR_400HZ= 1
    ODR_200HZ= 2
    ODR_100HZ= 3
    ODR_50HZ= 4
    ODR_12_5HZ= 5
    ODR_6_25HZ= 6
    ODR_1_56HZ= 7

    FSR_2G= 0
    FSR_4G= 1
    FSR_8G= 2

# Constants for configuring the LTR329 ambient light sensor
class AmbientLightLtr329:
    GAIN_1X= 0
    GAIN_2X= 1
    GAIN_4X= 2
    GAIN_8X= 3
    GAIN_48X= 4
    GAIN_96X= 5

    INTEGRATION_TIME_100MS= 0
    INTEGRATION_TIME_50MS= 1
    INTEGRATION_TIME_200MS= 2
    INTEGRATION_TIME_400MS= 3
    INTEGRATION_TIME_150MS= 4
    INTEGRATION_TIME_250MS= 5
    INTEGRATION_TIME_300MS= 6
    INTEGRATION_TIME_350MS= 7

    MEASUREMENT_RATE_50MS= 0
    MEASUREMENT_RATE_100MS= 1
    MEASUREMENT_RATE_200MS= 2
    MEASUREMENT_RATE_500MS= 3
    MEASUREMENT_RATE_1000MS= 4
    MEASUREMENT_RATE_2000MS= 5

# Constants for configuring the Bosch barometers
class BarometerBosch:
    OVERSAMPLING_SKIP= 0,
    OVERSAMPLING_ULTRA_LOW_POWER= 1
    OVERSAMPLING_LOW_POWER= 2
    OVERSAMPLING_STANDARD= 3
    OVERSAMPLING_HIGH= 4
    OVERSAMPLING_ULTRA_HIGH= 5

    IIR_FILTER_OFF= 0
    IIR_FILTER_AVG_2= 1
    IIR_FILTER_AVG_4= 2
    IIR_FILTER_AVG_8= 3
    IIR_FILTER_AVG_16= 4

class BarometerBmp280:
    STANDBY_TIME_0_5MS= 0
    STANDBY_TIME_62_5MS= 1
    STANDBY_TIME_125MS= 2
    STANDBY_TIME_250MS= 3
    STANDBY_TIME_500MS= 4
    STANDBY_TIME_1000MS= 5
    STANDBY_TIME_2000MS= 6
    STANDBY_TIME_4000MS= 7

class BarometerBme280:
    STANDBY_TIME_0_5MS= 0
    STANDBY_TIME_62_5MS= 1
    STANDBY_TIME_125MS= 2
    STANDBY_TIME_250MS= 3
    STANDBY_TIME_500MS= 4
    STANDBY_TIME_1000MS= 5
    STANDBY_TIME_10MS= 6
    STANDBY_TIME_20MS= 7

# Constants for configuring the BMI160 gyro
class GyroBmi160:
    ROTATION_X_AXIS_INDEX = 0
    ROTATION_Y_AXIS_INDEX = 1
    ROTATION_Z_AXIS_INDEX = 2

    ODR_25HZ= 6
    ODR_50HZ= 7
    ODR_100HZ= 8
    ODR_200HZ= 9
    ODR_400HZ= 10
    ODR_800HZ= 11
    ODR_1600HZ= 12
    ODR_3200HZ= 13

    FSR_2000DPS= 0
    FSR_1000DPS= 1
    FSR_500DPS= 2
    FSR_250DPS= 3
    FSR_125DPS= 4

class MultiChannelTemperature:
    SOURCE_INVALID= -1
    SOURCE_NRF_DIE= 0
    SOURCE_EXT_THERM= 1
    SOURCE_BMP280= 2
    SOURCE_PRESET_THERM= 3

    METAWEAR_R_CHANNEL_ON_DIE= 0
    METAWEAR_R_CHANNEL_EXT_THERMISTOR= 1

    METAWEAR_RPRO_CHANNEL_ON_DIE= 0
    METAWEAR_RPRO_CHANNEL_ON_BOARD_THERMISTOR= 1
    METAWEAR_RPRO_CHANNEL_EXT_THERMISTOR= 2
    METAWEAR_RPRO_CHANNEL_BMP280= 3

class Gpio:
    # Python wrapper for the MblMwGpioAnalogReadParameters struct
    class AnalogReadParameters(Structure):
        _fields_= [
            ("pullup_pin", c_ubyte),
            ("pulldown_pin", c_ubyte),
            ("virtual_pin", c_ubyte),
            ("delay_us", c_ushort)
        ]

    UNUSED_PIN = 0xff

    PULL_MODE_UP= 0
    PULL_MODE_DOWN= 1
    PULL_MODE_NONE= 2

    ANALOG_READ_MODE_ABS_REF= 0
    ANALOG_READ_MODE_ADC= 1

    PIN_CHANGE_TYPE_RISING= 1
    PIN_CHANGE_TYPE_FALLING= 2
    PIN_CHANGE_TYPE_ANY= 3

class MagnetometerBmm150:
    BFIELD_X_AXIS_INDEX = 0 
    BFIELD_Y_AXIS_INDEX = 1 
    BFIELD_Z_AXIS_INDEX = 2

    PRESET_LOW_POWER= 0
    PRESET_REGULAR= 1
    PRESET_ENHANCED_REGULAR= 2
    PRESET_HIGH_ACCURACY= 3

    ODR_10_HZ = 0
    ODR_2_HZ = 1
    ODR_6_HZ = 2
    ODR_8_HZ = 3
    ODR_15_HZ = 4
    ODR_20_HZ = 5
    ODR_25_HZ = 6
    ODR_30_HZ = 7

class ColorDetectorTcs34725:
    ADC_CLEAR_INDEX = 0 
    ADC_RED_INDEX = 1 
    ADC_GREEN_INDEX = 2
    ADC_BLUE_INDEX = 3

    GAIN_1X= 0
    GAIN_4X= 1
    GAIN_16X= 2
    GAIN_60X= 3

class ProximityTsl2671:
    CHANNEL_0= 1
    CHANNEL_1= 2
    CHANNEL_BOTH= 3

    CURRENT_100MA= 0
    CURRENT_50MA= 1
    CURRENT_25MA= 2
    CURRENT_12_5MA= 3

class HumidityBme280:
    OVERSAMPLING_1X= 1
    OVERSAMPLING_2X= 2
    OVERSAMPLING_4X= 3
    OVERSAMPLING_8X= 4
    OVERSAMPLING_16X= 5

class I2C:
    # Python wrapper for the MblMwI2cParameters struct
    class ReadParameters(Structure):
        _fields_= [
            ("device_addr", c_ubyte),
            ("register_addr", c_ubyte)
        ]

class SPI:
    # Python wrapper for the MblMwI2cParameters struct
    class Parameters(Structure):
        _fields_= [
            ("mode", c_int),
            ("frequency", c_int),
            ("data", c_void_p),
            ("data_length", c_ubyte),
            ("slave_select_pin", c_ubyte),
            ("clock_pin", c_ubyte),
            ("mosi_pin", c_ubyte),
            ("miso_pin", c_ubyte),
            ("lsb_first", c_ubyte),
            ("use_nrf_pins", c_ubyte),
        ]

    MODE_0= 0
    MODE_1= 1
    MODE_2= 2
    MODE_3= 3

    FREQUENCY_125_KHZ= 0
    FREQUENCY_250_KHZ= 1
    FREQUENCY_500_KHZ= 2
    FREQUENCY_1_MHZ= 3
    FREQUENCY_2_MHZ= 4
    FREQUENCY_4_MHZ= 5
    FREQUENCY_8_MHZ= 6

class SensorFusion:
    CALIBRATION_ACCURACY_UNRELIABLE = 0
    CALIBRATION_ACCURACY_LOW = 1
    CALIBRATION_ACCURACY_MEDIUM = 2
    CALIBRATION_ACCURACY_HIGH = 3

    MODE_SLEEP = 0
    MODE_NDOF = 1
    MODE_IMU_PLUS = 2
    MODE_COMPASS = 3
    MODE_M4G = 4

    ACC_RANGE_2G = 0
    ACC_RANGE_4G = 1
    ACC_RANGE_8G = 2
    ACC_RANGE_16G = 3

    GYRO_RANGE_2000DPS = 0
    GYRO_RANGE_1000DPS = 1
    GYRO_RANGE_500DPS = 2
    GYRO_RANGE_250DPS = 3

    DATA_CORRECTED_ACC = 0
    DATA_CORRECTED_GYRO = 1
    DATA_CORRECTED_MAG = 2
    DATA_QUATERION = 3
    DATA_EULER_ANGLE = 4
    DATA_GRAVITY_VECTOR = 5
    DATA_LINEAR_ACC = 6



from common import TestMetaWearBase
from ctypes import create_string_buffer
from mbientlab.metawear.core import CartesianFloat, CorrectedCartesianFloat, EulerAngle, Quaternion
from mbientlab.metawear.sensor import SensorFusion
import copy

class TestSensorFusion(TestMetaWearBase):
    def setUp(self):
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD

        super().setUp()

    def test_received_data(self):
        tests = [
            {
                'expected' : CorrectedCartesianFloat(x = -3.3799, y = 15.9995, z = -15.9995, accuracy = SensorFusion.CALIBRATION_ACCURACY_UNRELIABLE),
                'response' : create_string_buffer(b'\x19\x04\x20\x3e\x53\xc5\x0c\xfe\x79\x46\x0c\xfe\x79\xc6\x00', 15),
                'data' : SensorFusion.DATA_CORRECTED_ACC,
                'name': 'corrected acc'
            },
            {
                'expected' : CorrectedCartesianFloat(x = 72.669, y = -56.096, z = 528.820, accuracy = SensorFusion.CALIBRATION_ACCURACY_UNRELIABLE),
                'response' : create_string_buffer(b'\x19\x05\x7a\x56\x91\x42\xb4\x62\x60\xc2\x73\x34\x04\x44\x00', 15),
                'data' : SensorFusion.DATA_CORRECTED_GYRO,
                'name': 'corrected gyro'
            },
            {
                'expected' : CorrectedCartesianFloat(x = 32.500, y = -14.800, z = 29.700, accuracy = SensorFusion.CALIBRATION_ACCURACY_HIGH),
                'response' : create_string_buffer(b'\x19\x06\x00\x00\x02\x42\xcd\xcc\x6c\xc1\x9a\x99\xed\x41\x03', 15),
                'data' : SensorFusion.DATA_CORRECTED_MAG,
                'name': 'corrected mag'
            },
            {
                'expected' : Quaternion(w = 0.940, x = -0.050, y = -0.154, z = -0.301),
                'response' : create_string_buffer(b'\x19\x07\x1b\x9b\x70\x3f\x8c\x5e\x4d\xbd\x07\x7f\x1d\xbe\x78\x02\x9a\xbe', 18),
                'data' : SensorFusion.DATA_QUATERION,
                'name': 'quaternion'
            },
            {
                'expected' : EulerAngle(heading = 24.747, pitch = -120.862, yaw = 24.747, roll = -33.046),
                'response' : create_string_buffer(b'\x19\x08\xb1\xf9\xc5\x41\x44\xb9\xf1\xc2\x1a\x2f\x04\xc2\xb1\xf9\xc5\x41', 18),
                'data' : SensorFusion.DATA_EULER_ANGLE,
                'name': 'euler angle'
            },
            {
                'expected' : CartesianFloat(x = 0.042, y = 0.826, z = -0.562),
                'response' : create_string_buffer(b'\x19\x09\xee\x20\xd3\x3e\xb2\x93\x01\x41\x04\x59\xb0\xc0', 14),
                'data' : SensorFusion.DATA_GRAVITY_VECTOR,
                'name': 'gravity'
            },
            {
                'expected' : CartesianFloat(x = 0.296, y = 1.439, z = -0.380),
                'response' : create_string_buffer(b'\x19\x0a\x2f\xca\x39\x40\x86\xd4\x61\x41\x80\x4c\x6e\xc0', 14),
                'data' : SensorFusion.DATA_LINEAR_ACC,
                'name': 'linear acc'
            }
        ]

        for test in tests:
            with self.subTest(data = test['name']):
                signal = self.libmetawear.mbl_mw_sensor_fusion_get_data_signal(self.board, test['data'])
                self.libmetawear.mbl_mw_datasignal_subscribe(signal, self.sensor_data_handler)

                response = test['response']
                expected_value = test['expected']
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))

                self.assertEqual(self.data, expected_value)

    def test_sensor_control(self):
        data_sources = [
            SensorFusion.DATA_CORRECTED_ACC, SensorFusion.DATA_CORRECTED_GYRO, SensorFusion.DATA_CORRECTED_MAG,
            SensorFusion.DATA_QUATERION, SensorFusion.DATA_EULER_ANGLE,
            SensorFusion.DATA_GRAVITY_VECTOR, SensorFusion.DATA_LINEAR_ACC
        ]
        test_bases = [
            {
                'enable': 6,
                'mode': SensorFusion.MODE_NDOF,
                'name': 'ndof',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x13, 0x02, 0x01, 0x00],
                    [0x15, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x13, 0x01, 0x01],
                    [0x15, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x13, 0x01, 0x00],
                    [0x15, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x13, 0x02, 0x00, 0x01],
                    [0x15, 0x02, 0x00, 0x01]
                ]
            },
            {
                'enable': 4,
                'mode': SensorFusion.MODE_IMU_PLUS,
                'name': 'imu_plus',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x13, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x13, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x13, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x13, 0x02, 0x00, 0x01]
                ]
            },
            {
                'enable': 4,
                'mode': SensorFusion.MODE_COMPASS,
                'name': 'compass',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x15, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x15, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x15, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x15, 0x02, 0x00, 0x01]
                ]
            },
            {
                'enable': 4,
                'mode': SensorFusion.MODE_M4G,
                'name': 'm4g',
                'expected': [
                    [0x03, 0x02, 0x01, 0x00],
                    [0x15, 0x02, 0x01, 0x00],
                    [0x03, 0x01, 0x01],
                    [0x15, 0x01, 0x01],
                    [0x19, 0x03, 0x00, 0x00],
                    [0x19, 0x01, 0x01],
                    [0x19, 0x01, 0x00],
                    [0x19, 0x03, 0x00, 0x7f],
                    [0x03, 0x01, 0x00],
                    [0x15, 0x01, 0x00],
                    [0x03, 0x02, 0x00, 0x01],
                    [0x15, 0x02, 0x00, 0x01]
                ]
            }
        ]

        tests= []
        for base in test_bases:
            for source in data_sources:
                test = copy.deepcopy(base)
                test['expected'][test['enable']][2] |= (0x1 << source)
                test['source'] = source
                tests.append(test)

        for test in tests:
            with self.subTest(mode = test['name']):
                self.command_history = []
                self.libmetawear.mbl_mw_sensor_fusion_clear_enabled_mask(self.board)

                self.libmetawear.mbl_mw_sensor_fusion_set_mode(self.board, test['mode'])
                self.libmetawear.mbl_mw_sensor_fusion_enable_data(self.board, test['source'])
                self.libmetawear.mbl_mw_sensor_fusion_start(self.board)
                self.libmetawear.mbl_mw_sensor_fusion_stop(self.board)

                self.assertEqual(test['expected'], self.command_history)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
discover
-----------

:copyright: 2016-11-29 by hbldh <henrik.blidh@nedomkull.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import os
import signal
import subprocess
import time

from pymetawear.exceptions import PyMetaWearException

try:
    input_fcn = raw_input
except NameError:
    input_fcn = input


def discover_devices(timeout=5):
    """Discover Bluetooth Low Energy Devices nearby on Linux

    Using ``hcitool`` from Bluez in subprocess, which requires root privileges.
    However, ``hcitool`` can be allowed to do scan without elevated permission.

    Install linux capabilities manipulation tools:

    .. code-block:: bash

        $ sudo apt-get install libcap2-bin

    Sets the missing capabilities on the executable quite like the setuid bit:

    .. code-block:: bash

        $ sudo setcap 'cap_net_raw,cap_net_admin+eip' `which hcitool`

    **References:**

    * `StackExchange, hcitool without sudo <https://unix.stackexchange.com/questions/96106/bluetooth-le-scan-as-non-root>`_
    * `StackOverflow, hcitool lescan with timeout <https://stackoverflow.com/questions/26874829/hcitool-lescan-will-not-print-in-real-time-to-a-file>`_

    :param int timeout: Duration of scanning.
    :return: List of tuples with `(address, name)`.
    :rtype: list

    """
    p = subprocess.Popen(["hcitool", "lescan"], stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    time.sleep(timeout)
    os.kill(p.pid, signal.SIGINT)
    out, err = p.communicate()
    if len(out) == 0 and len(err) > 0:
        if err == b'Set scan parameters failed: Operation not permitted\n':
            raise PyMetaWearException("Missing capabilites for hcitool!")
        if err == b'Set scan parameters failed: Input/output error\n':
            raise PyMetaWearException("Could not perform scan.")
    ble_devices = list(set([tuple(x.split(' ')) for x in
                            filter(None, out.decode('utf8').split('\n')[1:])]))
    filtered_devices = {}
    for d in ble_devices:
        if d[0] not in filtered_devices:
            filtered_devices[d[0]] = d[1]
        else:
            if filtered_devices.get(d[0]) == '(unknown)':
                filtered_devices[d[0]] = d[1]

    return [(k, v) for k, v in filtered_devices.items()]


def select_device(timeout=3):
    """Run `discover_devices` and display a list to select from.

    :param int timeout: Duration of scanning.
    :return: The selected device's address.
    :rtype: str

    """
    print("Discovering nearby Bluetooth Low Energy devices...")
    ble_devices = discover_devices(timeout=timeout)
    if len(ble_devices) > 1:
        for i, d in enumerate(ble_devices):
            print("[{0}] - {1}: {2}".format(i + 1, *d))
        s = input("Which device do you want to connect to? ")
        if int(s) <= (i + 1):
            address = ble_devices[int(s) - 1][0]
        else:
            raise ValueError("Incorrect selection. Aborting...")
    elif len(ble_devices) == 1:
        address = ble_devices[0][0]
        print("Found only one device: {0}: {1}.".format(*ble_devices[0][::-1]))
    else:
        raise ValueError("Did not detect any BLE devices.")
    return address



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-16

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging
from ctypes import byref

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.modules.base import PyMetaWearModule
from pymetawear.mbientlab.metawear.peripheral import Led

log = logging.getLogger(__name__)


class LEDModule(PyMetaWearModule):
    """MetaWear Switch module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, debug=False):
        super(LEDModule, self).__init__(board, debug)

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}".format(self.module_name)

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "LED"

    def notifications(self, callback=None):
        """No subscriptions possible for LED module.

        :raises: :py:exc:`~PyMetaWearException`

        """
        raise PyMetaWearException(
            "No notifications available for LED module.")

    def load_preset_pattern(self, preset_name, **kwargs):
        """Loads a preset configuration.

        :param Led.Pattern pattern: One of the strings `blink`, `pulse` or `solid`.
        :param str preset_name: One of the strings `blink`, `pulse` or `solid`.
        :return: The preset pattern.
        :rtype: :py:class:`pymetawear.mbientlab.metawear.peripheral.Led.Pattern`

        """
        preset = self._get_preset(preset_name.lower())
        pattern = Led.Pattern(**kwargs)
        libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), preset)
        return pattern

    def write_pattern(self, pattern, color):
        """Writes the led pattern to the board.

        :param Led.Pattern pattern: The pattern to write to board.
        :param str color: `g`, `b` or 'r'

        """
        color = self._get_color(color.lower())
        libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), color)

    def play(self):
        """Executes any stored LED pattern."""
        libmetawear.mbl_mw_led_play(self.board)

    def autoplay(self):
        """Plays any programmed patterns, and immediately plays
        any patterns programmed later.
        """
        libmetawear.mbl_mw_led_autoplay(self.board)

    def pause(self):
        """Pause any playing LED pattern."""
        libmetawear.mbl_mw_led_pause(self.board)

    def stop(self):
        """Stop any playing LED pattern."""
        libmetawear.mbl_mw_led_stop(self.board)

    def stop_and_clear(self):
        """Stop any playing LED pattern. and clears
        all pattern cofigurations.
        """
        libmetawear.mbl_mw_led_stop_and_clear(self.board)

    def _get_color(self, s):
        return {
            'g': Led.COLOR_GREEN,
            'r': Led.COLOR_RED,
            'b': Led.COLOR_BLUE
        }.get(s, Led.COLOR_GREEN)

    def _get_preset(self, s):
        return {
            'blink': Led.PRESET_BLINK,
            'pulse': Led.PRESET_PULSE,
            'solid': Led.PRESET_SOLID
        }.get(s, Led.PRESET_BLINK)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
mock_backend
-----------

:copyright: 2016-11-25 by hbldh <henrik.blidh@nedomkull.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import

import uuid
from ctypes import create_string_buffer

from pymetawear.specs import METAWEAR_SERVICE_NOTIFY_CHAR, \
    DEV_INFO_FIRMWARE_CHAR, DEV_INFO_MODEL_CHAR
from pymetawear.backends import BLECommunicationBackend


UUID2HANDLES = {
    uuid.UUID("326a9001-85cb-9195-d9dd-464cfbbae75a"): 0x001d,
    METAWEAR_SERVICE_NOTIFY_CHAR[1]: 0x001d,
    DEV_INFO_FIRMWARE_CHAR[1]: 0x0014,
    DEV_INFO_MODEL_CHAR[1]: 0x001a
}


class MockBackend(BLECommunicationBackend):

    METAWEAR_R_BOARD = 0
    METAWEAR_RG_BOARD = 1
    METAWEAR_RPRO_BOARD = 2
    METAWEAR_CPRO_BOARD = 3
    METAWEAR_ENV_BOARD = 4
    METAWEAR_DETECT_BOARD = 5
    METAWEAR_MOTION_R_BOARD = 6

    boardType = 0

    def __init__(self, address, interface=None, timeout=None, debug=False):
        self.responses = []
        self.written_data = {}
        self.full_history = []
        self.command_history = []

        self.eventId = 0
        self.timerId = 0
        self.dataprocId = 0
        self.loggerId = 0
        self.timerSignals = []

        self.metawear_rg_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2D\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80', 2),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80', 2),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4)
        }

        self.metawear_r_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x00\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x01', 6),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1C', 5),
            0x0a: create_string_buffer(b'\x0A\x80\x00\x00\x1C', 5),
            0x0b: create_string_buffer(b'\x0B\x80\x00\x02\x08\x80\x31\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0C\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0D\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0F\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80', 2),
            0x13: create_string_buffer(b'\x13\x80', 2),
            0x14: create_string_buffer(b'\x14\x80', 2),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xFE\x80\x00\x00', 4)
        }

        self.metawear_rpro_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1C', 5),
            0x0a: create_string_buffer(b'\x0A\x80\x00\x00\x1C', 5),
            0x0b: create_string_buffer(b'\x0B\x80\x00\x02\x08\x80\x2D\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0C\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0D\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0F\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80\x00\x00', 4),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xFE\x80\x00\x00', 4)
        }

        self.metawear_cpro_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x00', 4),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1C', 5),
            0x0a: create_string_buffer(b'\x0A\x80\x00\x00\x1C', 5),
            0x0b: create_string_buffer(b'\x0B\x80\x00\x02\x08\x80\x2B\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0C\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0D\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0F\x80\x00\x00', 4),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x00', 4),
            0x12: create_string_buffer(b'\x12\x80\x00\x00', 4),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80\x00\x00', 4),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xFE\x80\x00\x00', 4)
        }

        self.metawear_detector_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x03\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x01\x03\x03\x03\x03\x01', 9),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2b\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x01\x08', 5),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x03', 4),
            0x12: create_string_buffer(b'\x12\x80', 2),
            0x13: create_string_buffer(b'\x13\x80', 2),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80\x00\x00', 4),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4),
        }

        self.metawear_environment_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x03\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x01\x03\x03\x03\x03\x01', 9),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2b\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x00', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x01\x08', 5),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x03', 4),
            0x12: create_string_buffer(b'\x12\x80\x01\x00', 4),
            0x13: create_string_buffer(b'\x13\x80', 2),
            0x14: create_string_buffer(b'\x14\x80', 2),
            0x15: create_string_buffer(b'\x15\x80', 2),
            0x16: create_string_buffer(b'\x16\x80\x00\x00', 4),
            0x17: create_string_buffer(b'\x17\x80\x00\x00', 4),
            0x18: create_string_buffer(b'\x18\x80', 4),
            0x19: create_string_buffer(b'\x19\x80', 2),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4),
        }
        self.metawear_motion_r_services = {
            0x01: create_string_buffer(b'\x01\x80\x00\x00', 4),
            0x02: create_string_buffer(b'\x02\x80\x00\x00', 4),
            0x03: create_string_buffer(b'\x03\x80\x01\x01', 4),
            0x04: create_string_buffer(b'\x04\x80\x01\x00\x00\x03\x01\x02', 8),
            0x05: create_string_buffer(b'\x05\x80\x00\x01\x03\x03\x03\x03\x01', 9),
            0x06: create_string_buffer(b'\x06\x80\x00\x00', 4),
            0x07: create_string_buffer(b'\x07\x80\x00\x00', 4),
            0x08: create_string_buffer(b'\x08\x80\x00\x00', 4),
            0x09: create_string_buffer(b'\x09\x80\x00\x00\x1c', 5),
            0x0a: create_string_buffer(b'\x0a\x80\x00\x00\x1c', 5),
            0x0b: create_string_buffer(b'\x0b\x80\x00\x02\x08\x80\x2b\x00\x00', 9),
            0x0c: create_string_buffer(b'\x0c\x80\x00\x00\x08', 5),
            0x0d: create_string_buffer(b'\x0d\x80\x00\x01', 4),
            0x0f: create_string_buffer(b'\x0f\x80\x00\x01\x08', 5),
            0x10: create_string_buffer(b'\x10\x80', 2),
            0x11: create_string_buffer(b'\x11\x80\x00\x03', 4),
            0x12: create_string_buffer(b'\x12\x80\x00\x00', 4),
            0x13: create_string_buffer(b'\x13\x80\x00\x01', 4),
            0x14: create_string_buffer(b'\x14\x80\x00\x00', 4),
            0x15: create_string_buffer(b'\x15\x80\x00\x01', 4),
            0x16: create_string_buffer(b'\x16\x80', 2),
            0x17: create_string_buffer(b'\x17\x80', 2),
            0x18: create_string_buffer(b'\x18\x80', 2),
            0x19: create_string_buffer(b'\x19\x80\x00\x00\x03\x00\x06\x00\x02\x00\x01\x00', 12),
            0xfe: create_string_buffer(b'\xfe\x80\x00\x00', 4),
        }

        self.firmware_revision = create_string_buffer(b'1.1.3', 5)
        self._connected = False

        super(MockBackend, self).__init__(address, interface, timeout, debug)

    @property
    def is_connected(self):
        return self._connected

    def connect(self, clean_connect=False):
        self._connected = True
        super(MockBackend, self).connect(clean_connect)

    def disconnect(self):
        self._connected = False

    @property
    def requester(self):
        """Not used in MockBackend"""
        return None

    def read_gatt_char_by_uuid(self, characteristic):
        if isinstance(characteristic,  uuid.UUID):
            if characteristic == DEV_INFO_FIRMWARE_CHAR[1]:
                return self.firmware_revision.raw
            elif characteristic == DEV_INFO_MODEL_CHAR[1]:
                if (self.boardType == self.METAWEAR_RG_BOARD):
                    model_number = create_string_buffer(b'1', 1)
                elif (self.boardType == self.METAWEAR_R_BOARD):
                    model_number = create_string_buffer(b'0', 1)
                elif (self.boardType == self.METAWEAR_RPRO_BOARD):
                    model_number = create_string_buffer(b'1', 1)
                elif (self.boardType == self.METAWEAR_CPRO_BOARD or
                              self.boardType == self.METAWEAR_DETECT_BOARD or
                              self.boardType == self.METAWEAR_ENV_BOARD):
                    model_number = create_string_buffer(b'2', 1)
                elif (self.boardType == self.METAWEAR_MOTION_R_BOARD):
                    model_number = create_string_buffer(b'5', 1)
                return model_number.raw
        else:
            if (characteristic.contents.uuid_high == 0x00002a2400001000 and characteristic.contents.uuid_low == 0x800000805f9b34fb):
                if (self.boardType == self.METAWEAR_RG_BOARD):
                    model_number= create_string_buffer(b'1', 1)
                elif (self.boardType == self.METAWEAR_R_BOARD):
                    model_number= create_string_buffer(b'0', 1)
                elif (self.boardType == self.METAWEAR_RPRO_BOARD):
                    model_number= create_string_buffer(b'1', 1)
                elif (self.boardType == self.METAWEAR_CPRO_BOARD or self.boardType == self.METAWEAR_DETECT_BOARD or self.boardType == self.METAWEAR_ENV_BOARD):
                    model_number= create_string_buffer(b'2', 1)
                elif (self.boardType == self.METAWEAR_MOTION_R_BOARD):
                    model_number= create_string_buffer(b'5', 1)

                self._log("Read", characteristic, model_number.raw, len(model_number.raw, ))
                return model_number.raw

            elif (characteristic.contents.uuid_high == 0x00002a2600001000 and characteristic.contents.uuid_low == 0x800000805f9b34fb):
                self._log("Read", characteristic, self.firmware_revision.raw, len(self.firmware_revision.raw))
                return self.firmware_revision.raw

    def write_gatt_char_by_uuid(self, characteristic, command, length):
        characteristic_uuid = self.get_uuid(characteristic)

        self.command = []
        for i in range(0, length):
            self.command.append(command[i])

        self.full_history.append(self.command)
        if command[1] == 0x80:
            if (self.boardType == self.METAWEAR_RG_BOARD and
                        command[0] in self.metawear_rg_services):
                service_response = self.metawear_rg_services[command[0]]
            elif (self.boardType == self.METAWEAR_R_BOARD and
                          command[0] in self.metawear_r_services):
                service_response = self.metawear_r_services[command[0]]
            elif (self.boardType == self.METAWEAR_RPRO_BOARD and
                          command[0] in self.metawear_rpro_services):
                service_response = self.metawear_rpro_services[command[0]]
            elif (self.boardType == self.METAWEAR_CPRO_BOARD and
                          command[0] in self.metawear_cpro_services):
                service_response = self.metawear_cpro_services[command[0]]
            elif (self.boardType == self.METAWEAR_DETECT_BOARD and
                          command[0] in self.metawear_detector_services):
                service_response = self.metawear_detector_services[command[0]]
            elif (self.boardType == self.METAWEAR_ENV_BOARD and
                          command[0] in self.metawear_environment_services):
                service_response = self.metawear_environment_services[
                    command[0]]
            elif (self.boardType == self.METAWEAR_MOTION_R_BOARD and
                          command[0] in self.metawear_motion_r_services):
                service_response = self.metawear_motion_r_services[command[0]]
            self.handle_notify_char_output(self.get_handle(characteristic_uuid),
                                           service_response.raw)
        elif (command[0] == 0xb and command[1] == 0x84):
            reference_tick = create_string_buffer(
                b'\x0b\x84\x15\x04\x00\x00\x05', 7)
            self.handle_notify_char_output(self.get_handle(characteristic_uuid),
                                           reference_tick.raw)
        else:
            # ignore module discovey commands
            self.command_history.append(self.command)
            if (command[0] == 0xc and command[1] == 0x2):
                response = create_string_buffer(b'\x0c\x02', 3)
                response[2] = self.timerId
                self.timerId += 1
                self.handle_notify_char_output(
                    self.get_handle(characteristic_uuid), response.raw)
            elif (command[0] == 0xa and command[1] == 0x3):
                response = create_string_buffer(b'\x0a\x02', 3)
                response[2] = self.eventId
                self.eventId += 1
                self.handle_notify_char_output(
                    self.get_handle(characteristic_uuid), response.raw)
            elif (command[0] == 0x9 and command[1] == 0x2):
                response = create_string_buffer(b'\x09\x02', 3)
                response[2] = self.dataprocId
                self.dataprocId += 1
                self.handle_notify_char_output(
                    self.get_handle(characteristic_uuid), response.raw)
            elif (command[0] == 0xb and command[1] == 0x2):
                response = create_string_buffer(b'\x0b\x02', 3)
                response[2] = self.loggerId
                self.loggerId += 1
                self.handle_notify_char_output(
                    self.get_handle(characteristic_uuid), response.raw)
            elif (command[0] == 0xb and command[1] == 0x85):
                response = create_string_buffer(b'\x0b\x85\x9e\x01\x00\x00', 6)
                self.handle_notify_char_output(
                    self.get_handle(characteristic_uuid), response.raw)

    def get_handle(self, uuid, notify_handle=False):
        """Get handle for a characteristic UUID.

        :param uuid.UUID uuid: The UUID to get handle of.
        :param bool notify_handle:
        :return: Integer handle corresponding to the input characteristic UUID.
        :rtype: int

        """
        # TODO: Is this even needed?
        return UUID2HANDLES.get(uuid, 1) + int(notify_handle)

    def _subscribe(self, characterisitic_uuid, callback):
        return

    def _response_2_string_buffer(self, response):
        return create_string_buffer(bytes(response), len(response))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`sensorfusion`
==================

Created by mgeorgi <marcus.georgi@kinemic.de>
Created on 2016-02-01

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient
from pymetawear.mbientlab.metawear.sensor import SensorFusion

address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))

def handle_quat(data):
    # Handle a (epoch_time, (w,x,y,z)) quaternion tuple.¬
    epoch = data[0]
    xyzaccu = data[1]
    print("QUAT [{0}] W {1}, X {2}, Y {3}, Z {4}".format(epoch, *xyzaccu))

def handle_notification(data):
    # Handle a (epoch_time, (x,y,z,accuracy)) corrected acc¬
    # tuple.¬
    epoch = data[0]
    xyzaccu = data[1]
    print("ACC [{0}] X: {1}, Y: {2}, Z: {3}".format(epoch, *xyzaccu[:-1]))

def handle_gyro(data):
    # Handle a (epoch_time, (x,y,z,accuracy)) corrected gyro¬
    # tuple.¬
    epoch = data[0]
    xyzaccu = data[1]
    print("GYRO [{0}] X: {1}, Y: {2}, Z: {3}".format(epoch, *xyzaccu[:-1]))


def handle_euler(data):
    # Handle a (epoch_time, (heading,pitch,roll,yaw)) euler angle tuple.¬
    epoch = data[0]
    xyzaccu = data[1]
    print("EULER [{0}] Heading: {1}, Pitch: {2}, Roll: {3}, Yaw: {4}".format(
        epoch, *xyzaccu))


print("Write Sensor Fusion settings...")
c.sensorfusion.set_mode(SensorFusion.MODE_NDOF)
c.sensorfusion.set_acc_range(SensorFusion.ACC_RANGE_8G)
c.sensorfusion.set_gyro_range(SensorFusion.GYRO_RANGE_1000DPS)
print("Set Time Processor to limit data rate to 50Hz for each channel")
c.sensorfusion.set_sample_delay(SensorFusion.DATA_CORRECTED_ACC, 20)
c.sensorfusion.set_sample_delay(SensorFusion.DATA_CORRECTED_GYRO, 20)
c.sensorfusion.set_sample_delay(SensorFusion.DATA_QUATERION, 20)
print("Subscribing to Sensor Fusion Quaternion signal notifications...")
#c.sensorfusion.notifications(euler_angle_callback=handle_euler)
c.sensorfusion.notifications(corrected_acc_callback=handle_notification,
                             quaternion_callback=handle_quat,
                             corrected_gyro_callback=handle_gyro)

time.sleep(20.0)

print("Unsubscribe to notification...")
c.sensorfusion.notifications()

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`gyroscope`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-26

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))

if c.ambient_light.is_present:
    def callback(data):
        """Handle ambient light notification callback."""
        print("Ambient Light: {0}".format(data))


    print("Write ambient light settings...")
    c.ambient_light.set_settings(gain=4, integration_time=200, measurement_rate=200)
    print("Subscribing to ambient light signal notifications...")
    c.ambient_light.notifications(callback)

    time.sleep(20.0)

    print("Unsubscribe to notification...")
    c.ambient_light.notifications(None)

    time.sleep(5.0)
else:
    print("Ambient light sensor is not present on this board.")

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`led`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from ctypes import POINTER, c_float, cast
import time

from pymetawear import libmetawear
from pymetawear.client import MetaWearClient
from pymetawear.discover import select_device
from pymetawear.mbientlab.metawear.core import DataTypeId, Fn_DataPtr

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))


def temperature_callback(data):
    if data.contents.type_id == DataTypeId.FLOAT:
        data_ptr = cast(data.contents.value, POINTER(c_float))
        print("Temperature read: {0:.2f} C".format(data_ptr.contents.value))
    else:
        raise RuntimeError('Incorrect data type id: ' + str(data.contents.type_id))


# On-Die temperature (channel 0) is always present.
channel = 0

# Subscribe to notifications
data_signal = libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal(c.board, channel)
fcn_dptr = Fn_DataPtr(temperature_callback)
libmetawear.mbl_mw_datasignal_subscribe(data_signal, fcn_dptr)

for i in range(5):
    libmetawear.mbl_mw_datasignal_read(data_signal)
    time.sleep(1.0)

# Unsubscribe to notifications
libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)

time.sleep(1.0)

c.disconnect()

